@@ -151,8 +151,4 @@ McdConnection *mcd_account_get_connection (McdAccount *account);
 gboolean mcd_account_check_request (McdAccount *account, GHashTable *request,
                                     GError **error);
 
-gboolean mcd_account_parameter_is_secret (McdAccount *self,
-                                              const gchar *name);
-
-void mcd_account_property_changed (McdAccount *account, const gchar *name);
 #endif
