@@ -47,7 +47,7 @@ G_GNUC_INTERNAL
 GHashTable *_mcd_channel_get_immutable_properties (McdChannel *channel);
 
 G_GNUC_INTERNAL
-GPtrArray *_mcd_channel_details_build_from_list (GList *channels);
+GPtrArray *_mcd_channel_details_build_from_list (const GList *channels);
 G_GNUC_INTERNAL
 void _mcd_channel_details_free (GPtrArray *channels);
 
@@ -58,7 +58,8 @@ GHashTable *_mcd_channel_get_requested_properties (McdChannel *channel);
 G_GNUC_INTERNAL
 const gchar *_mcd_channel_get_request_path (McdChannel *channel);
 G_GNUC_INTERNAL
-const GList *_mcd_channel_get_satisfied_requests (McdChannel *channel);
+const GList *_mcd_channel_get_satisfied_requests (McdChannel *channel,
+                                                  gint64 *get_latest_time);
 G_GNUC_INTERNAL
 guint64 _mcd_channel_get_request_user_action_time (McdChannel *channel);
 G_GNUC_INTERNAL
@@ -80,6 +81,10 @@ G_GNUC_INTERNAL void _mcd_channel_depart (McdChannel *channel,
                                           TpChannelGroupChangeReason reason,
                                           const gchar *message);
 
+/* NULL-safe for @channel; @verb is for debug */
+G_GNUC_INTERNAL gboolean _mcd_tp_channel_should_close (TpChannel *channel,
+                                                       const gchar *verb);
+
 G_END_DECLS
 #endif
 
