@@ -1,8 +1,9 @@
-/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 8 -*- */
+/* vi: set et sw=4 ts=8 cino=t0,(0: */
+/* -*- Mode: C; indent-tabs-mode: nil; c-basic-offset: 4; tab-width: 8 -*- */
 /*
  * This file is part of mission-control
  *
- * Copyright (C) 2007 Nokia Corporation. 
+ * Copyright (C) 2007-2009 Nokia Corporation.
  *
  * Contact: Naba Kumar  <naba.kumar@nokia.com>
  *
@@ -27,8 +28,7 @@
 
 #include <glib.h>
 #include <glib-object.h>
-#include <libmissioncontrol/mc-account.h>
-#include <libtelepathy/tp-connmgr.h>
+#include <telepathy-glib/connection-manager.h>
 
 #include "mcd-operation.h"
 
@@ -41,56 +41,51 @@ G_BEGIN_DECLS
 #define MCD_CONNECTION_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), MCD_TYPE_CONNECTION, McdConnectionClass))
 
 typedef struct _McdConnection McdConnection;
+typedef struct _McdConnectionPrivate McdConnectionPrivate;
 typedef struct _McdConnectionClass McdConnectionClass;
 
+#include "mcd-channel.h"
+
 struct _McdConnection
 {
     McdOperation parent;
+    McdConnectionPrivate *priv;
 };
 
 struct _McdConnectionClass
 {
     McdOperationClass parent_class;
+    gboolean (*need_dispatch) (McdConnection *connection,
+                               const GPtrArray *channels);
+    void (*_mc_reserved2) (void);
+    gboolean (*request_channel) (McdConnection *connection,
+                                 McdChannel *channel);
+    void (*_mc_reserved3) (void);
+    void (*_mc_reserved4) (void);
+    void (*_mc_reserved5) (void);
+    void (*_mc_reserved6) (void);
 };
 
-#include "mcd-presence-frame.h"
 #include "mcd-dispatcher.h"
 
 GType mcd_connection_get_type (void);
 
-McdConnection *mcd_connection_new (DBusGConnection * dbus_connection,
-				   const gchar * bus_name,
-				   TpConnMgr * tp_conn_mgr,
-				   McAccount * account,
-				   McdPresenceFrame * presence_frame,
-				   McdDispatcher *dispatcher);
+const gchar *mcd_connection_get_object_path (McdConnection *connection);
+const gchar *mcd_connection_get_name (McdConnection *connection);
 
 /* Return the connection's account */
-McAccount *mcd_connection_get_account (McdConnection * connection);
+McdAccount *mcd_connection_get_account (McdConnection * connection);
 
-TelepathyConnectionStatus mcd_connection_get_connection_status (McdConnection
-								* connection);
+TpConnection *mcd_connection_get_tp_connection (McdConnection *connection);
 
 gboolean mcd_connection_request_channel (McdConnection *connection,
-					 const struct mcd_channel_request *req,
-					 GError ** error);
+					 McdChannel *channel);
 
 gboolean mcd_connection_cancel_channel_request (McdConnection *connection,
 					       	guint operation_id,
 						const gchar *requestor_client_id,
 					       	GError **error);
 
-gboolean mcd_connection_get_telepathy_details (McdConnection * id,
-					       gchar ** ret_servname,
-					       gchar ** ret_objpath);
-
-gboolean mcd_connection_remote_avatar_changed (McdConnection *connection,
-					       guint contact_id,
-					       const gchar *token);
-void mcd_connection_account_changed (McdConnection *connection);
-void mcd_connection_restart (McdConnection *connection);
-
-void mcd_connection_connect (McdConnection *connection);
 void mcd_connection_close (McdConnection *connection);
 
 G_END_DECLS
