@@ -1,8 +1,10 @@
-/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 8 -*- */
+/* vi: set et sw=4 ts=8 cino=t0,(0: */
+/* -*- Mode: C; indent-tabs-mode: nil; c-basic-offset: 4; tab-width: 8 -*- */
 /*
  * This file is part of mission-control
  *
- * Copyright (C) 2007 Nokia Corporation. 
+ * Copyright (C) 2007-2009 Nokia Corporation.
+ * Copyright (C) 2008 Collabora Ltd.
  *
  * Contact: Naba Kumar  <naba.kumar@nokia.com>
  *
@@ -33,37 +35,31 @@
  * FIXME
  */
 
+#include <config.h>
+
 #include <stdlib.h>
+
+#include <telepathy-glib/debug.h>
+
 #include "mcd-debug.h"
 #include "mcd-operation.h"
 
-static gint mc_debug = 0;
+gint mcd_debug_level = 0;
 
-void
-mcd_debug_ref (gpointer obj, const gchar *filename, gint linenum)
+gpointer
+mcd_debug_ref (gpointer obj,
+               const gchar *filename G_GNUC_UNUSED,
+               gint linenum G_GNUC_UNUSED)
 {
-    /* Function reference untouchable by macro processing */
-    gpointer (*untouchable_ref) (gpointer object);
-    
-    untouchable_ref = g_object_ref;
-    if (mc_debug >= 2)
-	g_debug ("[%s:%d]: Referencing (%d) object %p of type %s",
-		 filename, linenum, G_OBJECT (obj)->ref_count,
-		 obj, G_OBJECT_TYPE_NAME(obj));
-    untouchable_ref (obj);
+    return g_object_ref (obj);
 }
 
 void
-mcd_debug_unref (gpointer obj, const gchar *filename, gint linenum)
+mcd_debug_unref (gpointer obj,
+                 const gchar *filename G_GNUC_UNUSED,
+                 gint linenum G_GNUC_UNUSED)
 {
-    void (*untouchable_unref) (gpointer object);
-    
-    untouchable_unref = g_object_unref;
-    if (mc_debug >= 2)
-	g_debug ("[%s:%d]: Unreferencing (%d) object %p of type %s",
-		 filename, linenum, G_OBJECT (obj)->ref_count, obj,
-		 G_OBJECT_TYPE_NAME(obj));
-    untouchable_unref (obj);
+    g_object_unref (obj);
 }
 
 static void
@@ -96,12 +92,27 @@ mcd_debug_print_tree_real (gpointer object, gint level)
     g_string_free (indent_str, TRUE);
 }
 
+/* We don't really have debug categories yet */
+
+typedef enum {
+    MCD_DEBUG_MISC = 1 << 0,
+    MCD_DEBUG_TREES = 1 << 1
+} McdDebugCategory;
+
+static GDebugKey const keys[] = {
+    { "misc", MCD_DEBUG_MISC },
+    { "trees", MCD_DEBUG_TREES },
+    { NULL, 0 }
+};
+
+static McdDebugCategory categories = 0;
+
 void
 mcd_debug_print_tree (gpointer object)
 {
     g_return_if_fail (MCD_IS_MISSION (object));
 
-    if (mc_debug >= 2)
+    if (categories & MCD_DEBUG_TREES)
     {
 	g_debug ("Object Hierarchy of object %p", object);
 	g_debug ("[");
@@ -113,14 +124,59 @@ mcd_debug_print_tree (gpointer object)
 void mcd_debug_init ()
 {
     gchar *mc_debug_str;
+    guint level;
 
     mc_debug_str = getenv ("MC_DEBUG");
+
     if (mc_debug_str)
-	mc_debug = atoi (mc_debug_str);
+    {
+        /* historically, MC_DEBUG was an integer; try that first */
+        level = atoi (mc_debug_str);
+
+        /* if it wasn't an integer; try interpreting it as a
+         * telepathy-glib-style flags-word */
+        if (level == 0)
+        {
+            categories = g_parse_debug_string (mc_debug_str, keys,
+                                               G_N_ELEMENTS (keys) - 1);
+            tp_debug_set_flags (mc_debug_str);
+
+            /* mcd-debug.h uses the value of mcd_debug_level directly, so
+             * we need to set it nonzero to get uncategorized messages */
+            if ((categories & MCD_DEBUG_MISC) != 0 && mcd_debug_level == 0)
+            {
+                mcd_debug_level = 1;
+            }
+        }
+        else
+        {
+            /* this is API, and will also try to set up categories from the
+             * level */
+            mcd_debug_set_level (level);
+        }
+    }
+
+    if (mcd_debug_level >= 1)
+        g_debug ("%s version %s", PACKAGE, VERSION);
 }
 
-inline gint mcd_debug_get_level ()
+void
+mcd_debug_set_level (gint level)
 {
-    return mc_debug;
+    mcd_debug_level = level;
+
+    if (level >= 1)
+    {
+        categories |= MCD_DEBUG_MISC;
+    }
+    else
+    {
+        categories = 0;
+    }
+
+    if (level >= 2)
+    {
+        categories |= MCD_DEBUG_TREES;
+    }
 }
 
