@@ -42,8 +42,6 @@
 
 #include <dbus/dbus-glib-lowlevel.h>
 
-#include "mission-control-plugins/mission-control-plugins.h"
-
 #include "client-registry.h"
 #include "mcd-signals-marshal.h"
 #include "mcd-account-priv.h"
@@ -58,7 +56,6 @@
 #include "mcd-dispatch-operation-priv.h"
 #include "mcd-handler-map-priv.h"
 #include "mcd-misc.h"
-#include "plugin-loader.h"
 
 #include <telepathy-glib/defs.h>
 #include <telepathy-glib/gtypes.h>
@@ -76,9 +73,6 @@
 #include <string.h>
 #include "sp_timestamp.h"
 
-#define CREATE_CHANNEL TP_IFACE_CONNECTION_INTERFACE_REQUESTS ".CreateChannel"
-#define ENSURE_CHANNEL TP_IFACE_CONNECTION_INTERFACE_REQUESTS ".EnsureChannel"
-
 #define MCD_DISPATCHER_PRIV(dispatcher) (MCD_DISPATCHER (dispatcher)->priv)
 
 static void dispatcher_iface_init (gpointer, gpointer);
@@ -115,16 +109,6 @@ typedef struct
     gboolean handled;
 } McdChannelRecover;
 
-typedef struct
-{
-  McdDispatcher *dispatcher;
-  gchar *account_path;
-  GHashTable *properties;
-  gint64 user_action_time;
-  gchar *preferred_handler;
-  gboolean ensure;
-} McdChannelRequestACL;
-
 struct _McdDispatcherPrivate
 {
     /* Dispatching contexts */
@@ -805,7 +789,7 @@ mcd_dispatcher_client_handling_channel_cb (McdClientProxy *client,
            object_path);
 
     _mcd_handler_map_set_path_handled (self->priv->handler_map,
-                                       object_path, unique_name, bus_name);
+                                       object_path, unique_name);
 }
 
 static void mcd_dispatcher_update_client_caps (McdDispatcher *self,
@@ -821,9 +805,6 @@ mcd_dispatcher_client_capabilities_changed_cb (McdClientProxy *client,
 static void mcd_dispatcher_client_gone_cb (McdClientProxy *client,
                                            McdDispatcher *self);
 
-static void mcd_dispatcher_client_needs_recovery_cb (McdClientProxy *client,
-                                                     McdDispatcher *self);
-
 static void
 mcd_dispatcher_discard_client (McdDispatcher *self,
                                McdClientProxy *client)
@@ -837,9 +818,6 @@ mcd_dispatcher_discard_client (McdDispatcher *self,
     g_signal_handlers_disconnect_by_func (client,
                                           mcd_dispatcher_client_gone_cb,
                                           self);
-
-    g_signal_handlers_disconnect_by_func (client,
-        mcd_dispatcher_client_needs_recovery_cb, self);
 }
 
 static void
@@ -849,39 +827,6 @@ mcd_dispatcher_client_gone_cb (McdClientProxy *client,
     mcd_dispatcher_discard_client (self, client);
 }
 
-static void
-mcd_dispatcher_client_needs_recovery_cb (McdClientProxy *client,
-                                         McdDispatcher *self)
-{
-    GList *channels =
-        _mcd_handler_map_get_handled_channels (self->priv->handler_map);
-    const GList *observer_filters;
-    GList *list;
-
-    DEBUG ("called");
-
-    observer_filters = _mcd_client_proxy_get_observer_filters (client);
-
-    for (list = channels; list; list = list->next)
-    {
-        TpChannel *channel = list->data;
-        GHashTable *properties;
-
-        properties = tp_channel_borrow_immutable_properties (channel);
-
-        if (_mcd_client_match_filters (properties, observer_filters,
-            FALSE))
-        {
-            const gchar *account_path =
-                _mcd_handler_map_get_channel_account (self->priv->handler_map,
-                    tp_proxy_get_object_path (channel));
-
-            _mcd_client_recover_observer (client, channel, account_path);
-        }
-
-    }
-}
-
 static void
 mcd_dispatcher_client_added_cb (McdClientRegistry *clients,
                                 McdClientProxy *client,
@@ -898,11 +843,6 @@ mcd_dispatcher_client_added_cb (McdClientRegistry *clients,
     g_signal_connect (client, "handler-capabilities-changed",
                       G_CALLBACK (mcd_dispatcher_client_capabilities_changed_cb),
                       self);
-
-    g_signal_connect (client, "need-recovery",
-                      G_CALLBACK (mcd_dispatcher_client_needs_recovery_cb),
-                      self);
-
 }
 
 static void
@@ -1167,9 +1107,6 @@ mcd_dispatcher_init (McdDispatcher * dispatcher)
     priv->operation_list_active = FALSE;
 
     priv->connections = g_hash_table_new (NULL, NULL);
-
-    /* idempotent, not guaranteed to have been called yet */
-    _mcd_plugin_loader_init ();
 }
 
 McdDispatcher *
@@ -1241,8 +1178,25 @@ no_more:    /* either no more filters, or no more channels */
 void
 mcd_dispatcher_context_forget_all (McdDispatcherContext *context)
 {
+    GList *list;
+
     g_return_if_fail (context);
-    _mcd_dispatch_operation_forget_channels (context->operation);
+
+    /* make a temporary copy, which is destroyed during the loop - otherwise
+     * we'll be trying to iterate over the list at the same time
+     * that mcd_mission_abort results in modifying it, which would be bad */
+    list = _mcd_dispatch_operation_dup_channels (context->operation);
+
+    while (list != NULL)
+    {
+        mcd_mission_abort (list->data);
+        g_object_unref (list->data);
+        list = g_list_delete_link (list, list);
+    }
+
+    /* There should now be none left (they all aborted) */
+    g_return_if_fail (_mcd_dispatch_operation_peek_channels
+                      (context->operation) == NULL);
 }
 
 /**
@@ -1258,8 +1212,20 @@ mcd_dispatcher_context_forget_all (McdDispatcherContext *context)
 void
 mcd_dispatcher_context_destroy_all (McdDispatcherContext *context)
 {
+    GList *list;
+
     g_return_if_fail (context);
-    _mcd_dispatch_operation_destroy_channels (context->operation);
+
+    list = _mcd_dispatch_operation_dup_channels (context->operation);
+
+    while (list != NULL)
+    {
+        _mcd_channel_undispatchable (list->data);
+        g_object_unref (list->data);
+        list = g_list_delete_link (list, list);
+    }
+
+    mcd_dispatcher_context_forget_all (context);
 }
 
 /**
@@ -1283,9 +1249,25 @@ mcd_dispatcher_context_close_all (McdDispatcherContext *context,
                                   TpChannelGroupChangeReason reason,
                                   const gchar *message)
 {
+    GList *list;
+
     g_return_if_fail (context);
-    _mcd_dispatch_operation_leave_channels (context->operation, reason,
-                                            message);
+
+    if (message == NULL)
+    {
+        message = "";
+    }
+
+    list = _mcd_dispatch_operation_dup_channels (context->operation);
+
+    while (list != NULL)
+    {
+        _mcd_channel_depart (list->data, reason, message);
+        g_object_unref (list->data);
+        list = g_list_delete_link (list, list);
+    }
+
+    mcd_dispatcher_context_forget_all (context);
 }
 
 /**
@@ -1309,7 +1291,7 @@ mcd_dispatcher_context_process (McdDispatcherContext * context, gboolean result)
 {
     if (!result)
     {
-        _mcd_dispatch_operation_destroy_channels (context->operation);
+        mcd_dispatcher_context_destroy_all (context);
     }
 
     mcd_dispatcher_context_proceed (context);
@@ -1835,55 +1817,43 @@ _mcd_dispatcher_reinvoke_handler (McdDispatcher *dispatcher,
 {
     GList *request_as_list;
     const gchar *handler_unique;
-    const gchar *well_known_name = NULL;
-    GStrv possible_handlers = NULL;
-    McdClientProxy *handler = NULL;
+    GStrv possible_handlers;
+    McdClientProxy *handler;
 
     request_as_list = g_list_append (NULL, request);
 
     /* the unique name (process) of the current handler */
     handler_unique = _mcd_handler_map_get_handler (
         dispatcher->priv->handler_map,
-        mcd_channel_get_object_path (request), &well_known_name);
+        mcd_channel_get_object_path (request));
 
-    if (well_known_name != NULL)
+    /* work out how to invoke that process - any of its well-known names
+     * will do */
+    possible_handlers = mcd_dispatcher_dup_possible_handlers (dispatcher,
+                                                              request_as_list,
+                                                              handler_unique);
+
+    if (possible_handlers == NULL || possible_handlers[0] == NULL)
     {
-        /* We know which Handler well-known name was responsible: if it
-         * still exists, we want to call HandleChannels on it */
-        handler = _mcd_client_registry_lookup (dispatcher->priv->clients,
-                                               well_known_name);
+        /* The process is still running (otherwise it wouldn't be in the
+         * handler map), but none of its well-known names is still
+         * interested in channels of that sort. Oh well, not our problem.
+         */
+        DEBUG ("process %s no longer interested in this channel, not "
+               "reinvoking", handler_unique);
+        mcd_dispatcher_finish_reinvocation (request);
+        goto finally;
     }
 
+    handler = _mcd_client_registry_lookup (dispatcher->priv->clients,
+                                           possible_handlers[0]);
+
     if (handler == NULL)
     {
-        /* Failing that, maybe the Handler it was dispatched to was temporary;
-         * try to pick another Handler that can deal with it, on the same
-         * unique name (i.e. in the same process) */
-        possible_handlers = mcd_dispatcher_dup_possible_handlers (dispatcher,
-            request_as_list, handler_unique);
-
-        if (possible_handlers == NULL || possible_handlers[0] == NULL)
-        {
-            /* The process is still running (otherwise it wouldn't be in the
-             * handler map), but none of its well-known names is still
-             * interested in channels of that sort. Oh well, not our problem.
-             */
-            DEBUG ("process %s no longer interested in this channel, not "
-                   "reinvoking", handler_unique);
-            mcd_dispatcher_finish_reinvocation (request);
-            goto finally;
-        }
-
-        handler = _mcd_client_registry_lookup (dispatcher->priv->clients,
-                                               possible_handlers[0]);
-
-        if (handler == NULL)
-        {
-            DEBUG ("Handler %s does not exist in client registry, not "
-                   "reinvoking", possible_handlers[0]);
-            mcd_dispatcher_finish_reinvocation (request);
-            goto finally;
-        }
+        DEBUG ("Handler %s does not exist in client registry, not "
+               "reinvoking", possible_handlers[0]);
+        mcd_dispatcher_finish_reinvocation (request);
+        goto finally;
     }
 
     /* This is deliberately not the same call as for normal dispatching,
@@ -1962,13 +1932,11 @@ _mcd_dispatcher_add_channel_request (McdDispatcher *dispatcher,
 
 void
 _mcd_dispatcher_recover_channel (McdDispatcher *dispatcher,
-                                 McdChannel *channel,
-                                 const gchar *account_path)
+                                 McdChannel *channel)
 {
     McdDispatcherPrivate *priv;
     const gchar *path;
     const gchar *unique_name;
-    const gchar *well_known_name = NULL;
     gboolean requested;
     TpChannel *tp_channel;
 
@@ -1985,8 +1953,7 @@ _mcd_dispatcher_recover_channel (McdDispatcher *dispatcher,
     tp_channel = mcd_channel_get_tp_channel (channel);
     g_return_if_fail (tp_channel != NULL);
 
-    unique_name = _mcd_handler_map_get_handler (priv->handler_map, path,
-                                                &well_known_name);
+    unique_name = _mcd_handler_map_get_handler (priv->handler_map, path);
 
     if (unique_name != NULL)
     {
@@ -1995,8 +1962,7 @@ _mcd_dispatcher_recover_channel (McdDispatcher *dispatcher,
         _mcd_channel_set_status (channel,
                                  MCD_CHANNEL_STATUS_DISPATCHED);
         _mcd_handler_map_set_channel_handled (priv->handler_map, tp_channel,
-                                              unique_name, well_known_name,
-                                              account_path);
+                                              unique_name);
     }
     else
     {
@@ -2101,86 +2067,6 @@ finally:
     g_object_unref (am);
 }
 
-static void
-dispatcher_channel_request_acl_cleanup (gpointer data)
-{
-    McdChannelRequestACL *crd = data;
-
-    DEBUG ("cleanup acl (%p)", data);
-
-    g_free (crd->account_path);
-    g_free (crd->preferred_handler);
-    g_hash_table_unref (crd->properties);
-    g_object_unref (crd->dispatcher);
-
-    g_slice_free (McdChannelRequestACL, crd);
-}
-
-static void
-dispatcher_channel_request_acl_success (DBusGMethodInvocation *context,
-                                        gpointer data)
-{
-    McdChannelRequestACL *crd = data;
-
-    DEBUG ("complete acl (%p)", crd);
-
-    dispatcher_request_channel (MCD_DISPATCHER (crd->dispatcher),
-                                crd->account_path,
-                                crd->properties,
-                                crd->user_action_time,
-                                crd->preferred_handler,
-                                context,
-                                crd->ensure);
-}
-
-static void
-free_gvalue (gpointer gvalue)
-{
-    GValue *gv = gvalue;
-
-    g_value_unset (gv);
-    g_slice_free (GValue, gv);
-}
-
-static void
-dispatcher_channel_request_acl_start (McdDispatcher *dispatcher,
-                                      const gchar *method,
-                                      const gchar *account_path,
-                                      GHashTable *requested_properties,
-                                      gint64 user_action_time,
-                                      const gchar *preferred_handler,
-                                      DBusGMethodInvocation *context,
-                                      gboolean ensure)
-{
-    McdChannelRequestACL *crd = g_slice_new0 (McdChannelRequestACL);
-    GValue *account = g_slice_new0 (GValue);
-    GHashTable *params =
-      g_hash_table_new_full (g_str_hash, g_str_equal, NULL, free_gvalue);
-
-    g_value_init (account, G_TYPE_STRING);
-    g_value_set_string (account, account_path);
-    g_hash_table_insert (params, "account-path", account);
-
-    crd->dispatcher = g_object_ref (dispatcher);
-    crd->account_path = g_strdup (account_path);
-    crd->preferred_handler = g_strdup (preferred_handler);
-    crd->properties = g_hash_table_ref (requested_properties);
-    crd->user_action_time = user_action_time;
-    crd->ensure = ensure;
-
-    DEBUG ("start %s.%s acl (%p)", account_path, method, crd);
-
-    mcp_dbus_acl_authorised_async (dispatcher->priv->dbus_daemon,
-                                   context,
-                                   DBUS_ACL_TYPE_METHOD,
-                                   method,
-                                   params,
-                                   dispatcher_channel_request_acl_success,
-                                   crd,
-                                   dispatcher_channel_request_acl_cleanup);
-
-}
-
 static void
 dispatcher_create_channel (TpSvcChannelDispatcher *iface,
                            const gchar *account_path,
@@ -2189,14 +2075,13 @@ dispatcher_create_channel (TpSvcChannelDispatcher *iface,
                            const gchar *preferred_handler,
                            DBusGMethodInvocation *context)
 {
-    dispatcher_channel_request_acl_start (MCD_DISPATCHER (iface),
-                                          CREATE_CHANNEL,
-                                          account_path,
-                                          requested_properties,
-                                          user_action_time,
-                                          preferred_handler,
-                                          context,
-                                          FALSE);
+    dispatcher_request_channel (MCD_DISPATCHER (iface),
+                                account_path,
+                                requested_properties,
+                                user_action_time,
+                                preferred_handler,
+                                context,
+                                FALSE);
 }
 
 static void
@@ -2207,14 +2092,13 @@ dispatcher_ensure_channel (TpSvcChannelDispatcher *iface,
                            const gchar *preferred_handler,
                            DBusGMethodInvocation *context)
 {
-    dispatcher_channel_request_acl_start (MCD_DISPATCHER (iface),
-                                          ENSURE_CHANNEL,
-                                          account_path,
-                                          requested_properties,
-                                          user_action_time,
-                                          preferred_handler,
-                                          context,
-                                          TRUE);
+    dispatcher_request_channel (MCD_DISPATCHER (iface),
+                                account_path,
+                                requested_properties,
+                                user_action_time,
+                                preferred_handler,
+                                context,
+                                TRUE);
 }
 
 static void
