@@ -2,7 +2,6 @@
  * This file is part of telepathy-mission-control
  *
  * Copyright (C) 2008 Nokia Corporation.
- * Copyright (C) 2010 Collabora Ltd.
  *
  * Contact: Pekka Pessi  <first.last@nokia.com>
  *
@@ -29,9 +28,9 @@
 
 #include <glib-object.h>
 
-#include <telepathy-glib/telepathy-glib.h>
-
-#include <libmcclient/_gen/interfaces.h>
+#include <telepathy-glib/dbus.h>
+#include <libmcclient/mc-account-manager.h>
+#include <libmcclient/mc-account.h>
 #include <libmcclient/mc-profile.h>
 
 static gchar *app_name;
@@ -50,7 +49,6 @@ show_help (gchar * err)
 	    "    %1$s update <account name> [(int|uint|bool|string):<key>=<value>|clear:key] ...\n"
 	    "    %1$s display <account name> <display name>\n"
 	    "    %1$s nick <account name> <nick name>\n"
-	    "    %1$s service <account name> <service name>\n"
 	    "    %1$s icon <account name> <icon name>\n"
 	    "    %1$s show <account name>\n"
 	    "    %1$s get <account name> [key...]\n"
@@ -78,8 +76,8 @@ union command {
     } common;
 
     union {
-	gboolean (*manager) (TpAccountManager *manager);
-	gboolean (*account) (TpAccount *account);
+	gboolean (*manager) (McAccountManager *manager);
+	gboolean (*account) (McAccount *account);
     } ready;
 
     struct {
@@ -102,7 +100,7 @@ union command {
     struct {
 	struct common common;
 	gchar const *name;
-    } display, nick, icon, service;
+    } display, nick, icon;
 
     struct {
 	struct common common;
@@ -118,11 +116,11 @@ union command {
 
 struct presence {
     TpConnectionPresenceType type;
-    gchar *status;
-    gchar *message;
+    gchar const *status;
+    gchar const *message;
 };
 
-static const char *strip_prefix (char const *string, char const *prefix)
+static char *startswith (char const *string, char const *prefix)
 {
     while (*prefix && *string)
 	if (*prefix++ != *string++)
@@ -130,19 +128,21 @@ static const char *strip_prefix (char const *string, char const *prefix)
     return *prefix == '\0' ? (char *)string : NULL;
 }
 
-static char *ensure_prefix (char const *string)
+char const account_prefix[] =  "/org/freedesktop/Telepathy/Account/";
+
+static char *prefix (char const *string)
 {
-    if (g_str_has_prefix (string, TP_ACCOUNT_OBJECT_PATH_BASE))
+    if (startswith (string, account_prefix))
 	return g_strdup (string);
-    return g_strdup_printf ("%s%s", TP_ACCOUNT_OBJECT_PATH_BASE, string);
+    return g_strdup_printf ("%s%s", account_prefix, string);
 }
 
-static const char *skip_prefix (char const *string)
+static char *strip (char const *string)
 {
-    const char *prefixed = strip_prefix (string, TP_ACCOUNT_OBJECT_PATH_BASE);
-    if (prefixed != NULL)
-	return prefixed;
-    return string;
+    char *prefixed = startswith (string, account_prefix);
+    if (prefixed)
+	return (char *)prefixed;
+    return (char *)string;
 }
 
 static void
@@ -175,34 +175,27 @@ set_param (GHashTable *parameters,
 	return FALSE;
 
     strv_type_key = g_strsplit (param_value, ":", 2);
-
-    if (strv_type_key == NULL ||
-        strv_type_key[0] == NULL ||
-        strv_type_key[1] == NULL ||
-        strv_type_key[2] != NULL)
-        goto CLEANUP;
-
+    if (strv_type_key[0] == NULL ||
+	strv_type_key[1] == NULL ||
+	strv_type_key[2] != NULL)
+	goto CLEANUP;
     type = strv_type_key[0];
     param = strv_type_key[1];
 
     if (clear)
     {
-        if (strcmp (type, "clear") == 0 || strcmp (type, "unset") == 0)
-        {
-            g_ptr_array_add (clear, g_strdup (param));
-            ret = TRUE;
-            goto CLEANUP;
-        }
+	if (strcmp (type, "clear") == 0 || strcmp (type, "unset") == 0)
+	{
+	    g_ptr_array_add (clear, g_strdup (param));
+	    return TRUE;
+	}
     }
 
     strv_param_value = g_strsplit (param, "=", 2);
-
-    if (strv_param_value == NULL ||
-        strv_param_value[0] == NULL ||
-        strv_param_value[1] == NULL ||
-        strv_param_value[2] != NULL)
-        goto CLEANUP;
-
+    if (strv_param_value[0] == NULL ||
+	strv_param_value[1] == NULL ||
+	strv_param_value[2] != NULL)
+	goto CLEANUP;
     key = strv_param_value[0];
     value = strv_param_value[1];
 
@@ -240,7 +233,9 @@ set_param (GHashTable *parameters,
 	g_free (gvalue);
 
 CLEANUP:
+    if (strv_param_value)
 	g_strfreev (strv_param_value);
+    if (strv_type_key)
 	g_strfreev (strv_type_key);
     return ret;
 }
@@ -295,13 +290,6 @@ show_presence (gchar const *what, struct presence *presence)
     presence->type, presence->message);
 }
 
-static void
-free_presence (struct presence *presence)
-{
-  g_free (presence->status);
-  g_free (presence->message);
-}
-
 static TpConnectionPresenceType
 get_presence_type_for_status(char const *status)
 {
@@ -419,28 +407,6 @@ getter_list_add(gchar const *name,
     g_datalist_id_set_data(&getter_list, id, getter);
 }
 
-static const char *
-account_get_normalized_name (TpAccount *account)
-{
-    return g_object_get_data (G_OBJECT (account), "NormalizedName");
-}
-
-static TpConnectionPresenceType
-account_get_automatic_presence (TpAccount *account,
-                                char **status, char **message)
-{
-    if (status != NULL)
-        *status = g_strdup (g_object_get_data (G_OBJECT (account),
-                            "AutomaticPresenceStatus"));
-
-    if (message != NULL)
-        *message = g_strdup (g_object_get_data (G_OBJECT (account),
-                             "AutomaticPresenceMessage"));
-
-    return GPOINTER_TO_UINT (g_object_get_data (G_OBJECT (account),
-                             "AutomaticPresenceType"));
-}
-
 static void
 getter_list_init(void)
 {
@@ -450,35 +416,36 @@ getter_list_init(void)
 
     init = 1;
 
-    getter_list_add("DisplayName", GET_STRING, tp_account_get_display_name);
-    getter_list_add("Icon", GET_STRING, tp_account_get_icon_name);
-    getter_list_add("Valid", GET_BOOLEAN, tp_account_is_valid);
-    getter_list_add("Enabled", GET_BOOLEAN, tp_account_is_enabled);
-    getter_list_add("Nickname", GET_STRING, tp_account_get_nickname);
+    getter_list_add("DisplayName", GET_STRING, mc_account_get_display_name);
+    getter_list_add("Icon", GET_STRING, mc_account_get_icon);
+    getter_list_add("Valid", GET_BOOLEAN, mc_account_is_valid);
+    getter_list_add("Enabled", GET_BOOLEAN, mc_account_is_enabled);
+    getter_list_add("Nickname", GET_STRING, mc_account_get_nickname);
     getter_list_add("ConnectAutomatically", GET_BOOLEAN,
-		    tp_account_get_connect_automatically);
-    getter_list_add("NormalizedName", GET_STRING, account_get_normalized_name);
+		    mc_account_connects_automatically);
+    getter_list_add("NormalizedName", GET_STRING,
+		    mc_account_get_normalized_name);
 
     getter_list_add("AutomaticPresenceType",
-		    GET_PRESENCE_TYPE, account_get_automatic_presence);
+		    GET_PRESENCE_TYPE, mc_account_get_automatic_presence);
     getter_list_add("AutomaticPresenceStatus",
-		    GET_PRESENCE_STATUS, account_get_automatic_presence);
+		    GET_PRESENCE_STATUS, mc_account_get_automatic_presence);
     getter_list_add("AutomaticPresenceMessage",
-		    GET_PRESENCE_MESSAGE, account_get_automatic_presence);
+		    GET_PRESENCE_MESSAGE, mc_account_get_automatic_presence);
 
     getter_list_add("RequestedPresenceType",
-		    GET_PRESENCE_TYPE, tp_account_get_requested_presence);
+		    GET_PRESENCE_TYPE, mc_account_get_requested_presence);
     getter_list_add("RequestedPresenceStatus",
-		    GET_PRESENCE_STATUS, tp_account_get_requested_presence);
+		    GET_PRESENCE_STATUS, mc_account_get_requested_presence);
     getter_list_add("RequestedPresenceMessage",
-		    GET_PRESENCE_MESSAGE, tp_account_get_requested_presence);
+		    GET_PRESENCE_MESSAGE, mc_account_get_requested_presence);
 
     getter_list_add("CurrentPresenceType",
-		    GET_PRESENCE_TYPE, tp_account_get_current_presence);
+		    GET_PRESENCE_TYPE, mc_account_get_current_presence);
     getter_list_add("CurrentPresenceStatus",
-		    GET_PRESENCE_STATUS, tp_account_get_current_presence);
+		    GET_PRESENCE_STATUS, mc_account_get_current_presence);
     getter_list_add("CurrentPresenceMessage",
-		    GET_PRESENCE_MESSAGE, tp_account_get_current_presence);
+		    GET_PRESENCE_MESSAGE, mc_account_get_current_presence);
 }
 
 static Getter *
@@ -492,22 +459,20 @@ getter_by_name(char *name)
 /* ====================================================================== */
 
 static gboolean
-command_list (TpAccountManager *manager)
+command_list (McAccountManager *manager)
 {
-    GList *accounts;
+    const gchar * const *accounts;
 
-    accounts = tp_account_manager_get_valid_accounts (manager);
+    accounts = mc_account_manager_get_valid_accounts (manager);
 
-    if (accounts != NULL) {
-	GList *ptr;
+    if (accounts) {
+	int i;
 
 	command.common.ret = 0;
 
-	for (ptr = accounts; ptr != NULL; ptr = ptr->next) {
-	    puts (skip_prefix (tp_proxy_get_object_path (ptr->data)));
+	for (i = 0; accounts[i]; i++) {
+	    puts (strip (accounts[i]));
 	}
-
-	g_list_free (accounts);
     }
 
     return FALSE;                 /* stop mainloop */
@@ -515,7 +480,7 @@ command_list (TpAccountManager *manager)
 
 
 static void
-callback_for_create_account (TpAccountManager *proxy,
+callback_for_create_account (TpProxy *proxy,
 			     gchar const *account,
 			     GError const *error,
 			     gpointer user_data,
@@ -523,7 +488,7 @@ callback_for_create_account (TpAccountManager *proxy,
 {
     if (error == NULL) {
 	command.common.ret = 0;
-	puts (skip_prefix (account));
+	puts (strip (account));
     }
     else {
 	fprintf (stderr, "%s: %s\n", app_name, error->message);
@@ -532,7 +497,7 @@ callback_for_create_account (TpAccountManager *proxy,
 }
 
 static gboolean
-command_add (TpAccountManager *manager)
+command_add (McAccountManager *manager)
 {
     GHashTable *properties;
     GValue v_profile = { 0 };
@@ -549,7 +514,7 @@ command_add (TpAccountManager *manager)
     }
 
     return NULL !=
-	tp_cli_account_manager_call_create_account
+	mc_cli_account_manager_call_create_account
 	    (manager, 25000,
 	     command.add.manager,
 	     command.add.protocol,
@@ -561,7 +526,7 @@ command_add (TpAccountManager *manager)
 }
 
 static void
-callback_for_update_parameters (TpAccount *proxy,
+callback_for_update_parameters (TpProxy *proxy,
                                 const gchar **unchanged G_GNUC_UNUSED,
                                 const GError *error,
                                 gpointer user_data,
@@ -578,7 +543,7 @@ callback_for_update_parameters (TpAccount *proxy,
 }
 
 static void
-callback_for_void (TpAccount *proxy,
+callback_for_void (TpProxy *proxy,
 		   const GError *error,
 		   gpointer user_data,
 		   GObject *weak_object)
@@ -593,80 +558,49 @@ callback_for_void (TpAccount *proxy,
     g_main_loop_quit (main_loop);
 }
 
-static void
-callback_for_async (GObject *account,
-		    GAsyncResult *res,
-		    gpointer user_data)
-{
-    gboolean (* finish_func) (TpAccount *, GAsyncResult *, GError **);
-    GError *error = NULL;
-
-    finish_func = user_data;
-
-    if (!finish_func (TP_ACCOUNT (account), res, &error)) {
-        fprintf (stderr, "%s %s: %s\n", app_name, command.common.name,
-                 error->message);
-        g_error_free (error);
-    }
-    else {
-        command.common.ret = 0;
-    }
-
-    g_main_loop_quit (main_loop);
-}
-
 static gboolean
-command_remove (TpAccount *account)
+command_remove (McAccount *account)
 {
     return NULL !=
-	tp_cli_account_call_remove (account, 25000,
+	mc_cli_account_call_remove (account, 25000,
 				    callback_for_void,
 				    NULL, NULL, NULL);
 }
 
 static gboolean
-command_show (TpAccount *account)
+command_show (McAccount *account)
 {
     gchar const *name;
-    const GHashTable *parameters;
+    GHashTable *parameters;
     GHashTableIter i[1];
     gpointer keyp, valuep;
     struct presence automatic, current, requested;
 
-    name = skip_prefix (command.common.account);
+    name = strip (command.common.account);
 
     show ("Account", name);
-    show ("Display Name", tp_account_get_display_name (account));
-    show ("Normalized", account_get_normalized_name (account));
-    show ("Enabled", tp_account_is_enabled (account) ? "enabled" : "disabled");
-    show ("Valid", tp_account_is_valid (account) ? "" : "false");
-    show ("Icon", tp_account_get_icon_name (account));
+    show ("Display Name", mc_account_get_display_name (account));
+    show ("Normalized", mc_account_get_normalized_name (account));
+    show ("Enabled", mc_account_is_enabled (account) ? "enabled" : "disabled");
+    show ("Valid", mc_account_is_valid (account) ? "" : "false");
+    show ("Icon", mc_account_get_icon (account));
     show ("Connects",
-	  tp_account_get_connect_automatically (account) ? "automatically" : NULL);
-    show ("Nickname", tp_account_get_nickname (account));
+	  mc_account_connects_automatically (account) ? "automatically" : NULL);
+    show ("Nickname", mc_account_get_nickname (account));
 
-    automatic.type = account_get_automatic_presence (account,
-                                                     &automatic.status,
-                                                     &automatic.message);
+    mc_account_get_automatic_presence (account, &automatic.type,
+				       &automatic.status, &automatic.message);
     show_presence ("Automatic", &automatic);
-    free_presence (&automatic);
-
-    current.type = tp_account_get_current_presence (account,
-                                                    &current.status,
-                                                    &current.message);
+    mc_account_get_current_presence (account, &current.type,
+				     &current.status, &current.message);
     show_presence ("Current", &current);
-    free_presence (&current);
-
-    requested.type = tp_account_get_requested_presence (account,
-                                                        &requested.status,
-                                                        &requested.message);
+    mc_account_get_requested_presence (account, &requested.type,
+				       &requested.status, &requested.message);
     show_presence ("Requested", &requested);
-    free_presence (&requested);
-
     puts ("");
-    parameters = tp_account_get_parameters (account);
+    parameters = mc_account_get_parameters (account);
 
-    for (g_hash_table_iter_init (i, (GHashTable *) parameters);
+    for (g_hash_table_iter_init (i, parameters);
 	 g_hash_table_iter_next (i, &keyp, &valuep);) {
 	show_param (keyp, valuep);
     }
@@ -677,20 +611,17 @@ command_show (TpAccount *account)
 }
 
 static gboolean
-command_connection (TpAccount *account)
+command_connection (McAccount *account)
 {
-    TpConnection *conn;
+    gchar const *name;
+    TpConnectionStatus status;
+    TpConnectionStatusReason reason;
 
-    conn = tp_account_get_connection (account);
-
-    if (conn != NULL) {
-        TpConnectionStatus status;
-        TpConnectionStatusReason reason;
-	const gchar *name;
-
-	name = tp_proxy_get_object_path (conn);
-	status = tp_account_get_connection_status (account, &reason);
+    name = mc_account_get_connection_path (account);
+    status = mc_account_get_connection_status (account);
+    reason = mc_account_get_connection_status_reason (account);
 
+    if (name && name[0]) {
 	printf("%s %s %s\n", name,
 	       connection_status_as_string(status),
 	       connection_status_reason_as_string(reason));
@@ -698,14 +629,14 @@ command_connection (TpAccount *account)
     }
     else {
 	fprintf(stderr, "%s: no connection\n",
-		skip_prefix (command.common.account));
+		strip (command.common.account));
     }
 
     return FALSE;
 }
 
 static gboolean
-command_get (TpAccount *account)
+command_get (McAccount *account)
 {
     GPtrArray *args = command.get.args;
     GHashTable *parameters = NULL;
@@ -717,10 +648,10 @@ command_get (TpAccount *account)
 	Getter *getter = g_ptr_array_index(args, i);
 
 	if (getter->function) {
-	    gchar const *(*getstring)(TpAccount *) = getter->function;
-	    gboolean (*getboolean)(TpAccount *) = getter->function;
-	    TpConnectionPresenceType (*getpresence)(TpAccount *,
-				gchar **status, gchar **message) =
+	    gchar const *(*getstring)(McAccount *) = getter->function;
+	    gboolean (*getboolean)(McAccount *) = getter->function;
+	    void (*getpresence)(McAccount *, TpConnectionPresenceType *type,
+				gchar const **status, gchar const **message) =
 		getter->function;
 
 	    if (getter->type == GET_STRING) {
@@ -733,18 +664,14 @@ command_get (TpAccount *account)
 		     getter->type == GET_PRESENCE_STATUS ||
 		     getter->type == GET_PRESENCE_MESSAGE) {
 		struct presence presence;
-
-		presence.type = getpresence(account, &presence.status,
+		getpresence(account, &presence.type, &presence.status,
 			    &presence.message);
-
 		if (getter->type == GET_PRESENCE_TYPE)
 		    printf("%u\n", presence.type);
 		else if (getter->type == GET_PRESENCE_STATUS)
 		    printf("\"%s\"\n", presence.status);
 		else
 		    printf("\"%s\"\n", presence.message);
-
-                free_presence (&presence);
 	    }
 	    else {
 	    }
@@ -754,7 +681,7 @@ command_get (TpAccount *account)
 	    gchar *value;
 
 	    if (parameters == NULL)
-		parameters = (GHashTable *) tp_account_get_parameters(account);
+		parameters = mc_account_get_parameters(account);
 
 	    gvalue = g_hash_table_lookup(parameters, getter->name);
 
@@ -776,80 +703,59 @@ command_get (TpAccount *account)
 }
 
 static gboolean
-command_enable (TpAccount *account)
-{
-    tp_account_set_enabled_async (account, TRUE,
-                                  callback_for_async,
-                                  tp_account_set_enabled_finish);
-
-    return TRUE;
-}
-
-static gboolean
-command_disable (TpAccount *account)
-{
-    tp_account_set_enabled_async (account, FALSE,
-                                  callback_for_async,
-                                  tp_account_set_enabled_finish);
-
-    return TRUE;
-}
-
-static gboolean
-command_display (TpAccount *account)
-{
-    tp_account_set_display_name_async (account, command.display.name,
-                                       callback_for_async,
-                                       tp_account_set_display_name_finish);
-
-    return TRUE;
-}
-
-static gboolean
-command_nick (TpAccount *account)
-{
-    tp_account_set_nickname_async (account, command.nick.name,
-				   callback_for_async,
-                                   tp_account_set_nickname_finish);
-
-    return TRUE;
-}
-
-static gboolean
-command_service (TpAccount *account)
-{
-    tp_account_set_service_async (account, command.service.name,
-                                  callback_for_async,
-                                  tp_account_set_service_finish);
-
-    return TRUE;
-}
-
-static gboolean
-command_icon (TpAccount *account)
-{
-    tp_account_set_icon_name_async (account, command.icon.name,
-                                    callback_for_async,
-                                    tp_account_set_icon_name_finish);
-
-    return TRUE;
-}
-
-static gboolean
-command_auto_connect (TpAccount *account)
-{
-    tp_account_set_connect_automatically_async (account, command.boolean.value,
-                                                callback_for_async,
-                                                tp_account_set_connect_automatically_finish);
-
-    return TRUE;
-}
-
-static gboolean
-command_update (TpAccount *account)
+command_enable (McAccount *account)
 {
     return NULL !=
-	tp_cli_account_call_update_parameters (account, 25000,
+	mc_account_set_enabled (account, TRUE,
+				callback_for_void, NULL, NULL, NULL);
+}
+
+static gboolean
+command_disable (McAccount *account)
+{
+    return NULL !=
+	mc_account_set_enabled (account, FALSE,
+				callback_for_void, NULL, NULL, NULL);
+}
+
+static gboolean
+command_display (McAccount *account)
+{
+    return NULL !=
+	mc_account_set_display_name (account, command.display.name,
+				     callback_for_void, NULL, NULL, NULL);
+}
+
+static gboolean
+command_nick (McAccount *account)
+{
+    return NULL !=
+	mc_account_set_nickname (account, command.nick.name,
+				 callback_for_void, NULL, NULL, NULL);
+}
+
+static gboolean
+command_icon (McAccount *account)
+{
+    return NULL !=
+	mc_account_set_icon (account, command.icon.name,
+			     callback_for_void, NULL, NULL, NULL);
+}
+
+static gboolean
+command_auto_connect (McAccount *account)
+{
+    return NULL !=
+	mc_account_set_connect_automatically (account, command.boolean.value,
+					      callback_for_void,
+					      NULL, NULL, NULL);
+}
+
+static gboolean
+command_update (McAccount *account)
+{
+    return NULL !=
+	mc_cli_account_call_update_parameters (account, 25000,
 					       command.update.set,
 					       (const gchar  **)
 					       command.update.unset->pdata,
@@ -858,39 +764,27 @@ command_update (TpAccount *account)
 }
 
 static gboolean
-command_auto_presence (TpAccount *account)
+command_auto_presence (McAccount *account)
 {
-    GValue value = { 0, };
-    GValueArray *va = tp_value_array_build (3,
-                    G_TYPE_UINT, command.presence.type,
-					G_TYPE_STRING, command.presence.status,
-					G_TYPE_STRING, command.presence.message,
-                    G_TYPE_INVALID);
-
-    g_value_init (&value, TP_STRUCT_TYPE_SIMPLE_PRESENCE);
-    g_value_take_boxed (&value, va);
-
-	tp_cli_dbus_properties_call_set (account, 25000,
-                    TP_IFACE_ACCOUNT, "AutomaticPresence", &value,
-                    (tp_cli_dbus_properties_callback_for_set) callback_for_void,
-                    NULL, NULL, NULL);
-
-    g_value_unset (&value);
-
-    return TRUE;
+    return NULL !=
+	mc_account_set_automatic_presence (account,
+					   command.presence.type,
+					   command.presence.status,
+					   command.presence.message,
+					   callback_for_void,
+					   NULL, NULL, NULL);
 }
 
 static gboolean
-command_request (TpAccount *account)
+command_request (McAccount *account)
 {
-    tp_account_request_presence_async (account,
-                                       command.presence.type,
-                                       command.presence.status,
-                                       command.presence.message,
-                                       callback_for_async,
-                                       tp_account_request_presence_finish);
-
-    return TRUE;
+    return NULL !=
+	mc_account_set_requested_presence (account,
+					   command.presence.type,
+					   command.presence.status,
+					   command.presence.message,
+					   callback_for_void,
+					   NULL, NULL, NULL);
 }
 
 static void
@@ -998,9 +892,9 @@ parse (int argc, char **argv)
 	for (i = 3; argv[i]; i++) {
 	    char *name = argv[i];
 	    Getter *getter;
-	    const char *param = strip_prefix (name, "param=");
+	    char *param = startswith(name, "param=");
 
-	    if (param != NULL) {
+	    if (param) {
 		getter = g_new0(Getter, 1);
 		getter->name = param;
 		getter->type = GET_PARAM;
@@ -1066,16 +960,6 @@ parse (int argc, char **argv)
 	command.common.account = argv[2];
 	command.nick.name = argv[3];
     }
-    else if (strcmp (argv[1], "service") == 0)
-    {
-        /* Set service */
-	if (argc != 4)
-	    show_help ("Invalid service command.");
-
-	command.ready.account = command_service;
-	command.common.account = argv[2];
-	command.service.name = argv[3];
-    }
     else if (strcmp (argv[1], "icon") == 0)
     {
 	/* Set icon */
@@ -1188,88 +1072,33 @@ parse (int argc, char **argv)
 }
 
 static
-void manager_ready (GObject *manager,
-		    GAsyncResult *res,
+void manager_ready (McAccountManager *manager,
+		    const GError *error,
 		    gpointer user_data)
 {
-    GError *error = NULL;
-
-    if (!tp_proxy_prepare_finish (manager, res, &error)) {
+    if (error) {
 	fprintf (stderr, "%s: %s\n", app_name, error->message);
-	g_error_free (error);
     }
     else {
-	if (command.ready.manager (TP_ACCOUNT_MANAGER (manager)))
+	if (command.ready.manager (manager))
 	    return;
     }
 
     g_main_loop_quit (main_loop);
 }
 
-static void
-account_got_all_properties (TpProxy *account,
-                            GHashTable *properties,
-                            const GError *error,
-                            gpointer user_data,
-                            GObject *weak_object)
-{
-    TpConnectionPresenceType type;
-    const char *status, *message;
-
-    if (error != NULL)
-    {
-        fprintf (stderr, "%s: %s: %s\n",
-                 app_name, skip_prefix (command.common.account),
-                 error->message);
-    }
-
-    /* load properties not expose in TpAccount */
-    /* NormalizedName */
-    g_object_set_data_full (G_OBJECT (account), "NormalizedName",
-                            g_strdup (tp_asv_get_string (properties,
-                                                         "NormalizedName")),
-                            g_free);
-
-    /* AutomaticPresence */
-    tp_value_array_unpack (tp_asv_get_boxed (properties, "AutomaticPresence",
-                                             TP_STRUCT_TYPE_SIMPLE_PRESENCE),
-                           3, &type, &status, &message);
-
-    g_object_set_data (G_OBJECT (account), "AutomaticPresenceType",
-                       GUINT_TO_POINTER (type));
-    g_object_set_data_full (G_OBJECT (account), "AutomaticPresenceStatus",
-                            g_strdup (status), g_free);
-    g_object_set_data_full (G_OBJECT (account), "AutomaticPresenceMessage",
-                            g_strdup (message), g_free);
-
-    if (command.ready.account (TP_ACCOUNT (account)))
-        return;
-
-    g_main_loop_quit (main_loop);
-}
-
 static
-void account_ready (GObject *account,
-		    GAsyncResult *res,
+void account_ready (McAccount *account,
+		    const GError *error,
 		    gpointer user_data)
 {
-    GError *error = NULL;
-
-    if (!tp_proxy_prepare_finish (account, res, &error)) {
+    if (error) {
 	fprintf (stderr, "%s: %s: %s\n",
-		 app_name, skip_prefix (command.common.account),
-                 error->message);
-	g_error_free (error);
+		 app_name, strip (command.common.account), error->message);
     }
     else {
-        /* not all properties are exposed through TpAccount, request the
-         * remaining properties */
-        tp_cli_dbus_properties_call_get_all (account, 25000,
-                TP_IFACE_ACCOUNT,
-                account_got_all_properties,
-                NULL, NULL, NULL);
-
-        return;
+	if (command.ready.account (account))
+	    return;
     }
 
     g_main_loop_quit (main_loop);
@@ -1278,10 +1107,10 @@ void account_ready (GObject *account,
 int
 main (int argc, char **argv)
 {
-    TpAccountManager *am = NULL;
-    TpAccount *a = NULL;
-    TpDBusDaemon *dbus;
-    GError *error = NULL;
+    McAccountManager *am = NULL;
+    McAccount *a = NULL;
+    DBusGConnection *dbus_conn;
+    TpDBusDaemon *daemon;
 
     g_type_init ();
 
@@ -1291,43 +1120,24 @@ main (int argc, char **argv)
 
     command.common.ret = 1;
 
-    dbus = tp_dbus_daemon_dup (&error);
-    if (error != NULL) {
-        fprintf (stderr, "%s %s: Failed to connect to D-Bus: %s\n",
-            app_name, command.common.name, error->message);
-
-        g_error_free (error);
-
-        goto out;
-    }
+    dbus_conn = tp_get_bus ();
+    daemon = tp_dbus_daemon_new (dbus_conn);
+    dbus_g_connection_unref (dbus_conn);
 
     if (command.common.account == NULL) {
-        am = tp_account_manager_new (dbus);
-        tp_proxy_prepare_async (am, NULL, manager_ready, NULL);
+	am = mc_account_manager_new (daemon);
+	mc_account_manager_call_when_ready (am, manager_ready, NULL);
     }
     else {
-	command.common.account = ensure_prefix (command.common.account);
-	a = tp_account_new (dbus, command.common.account, &error);
-
-	if (error != NULL)
-	{
-	    fprintf (stderr, "%s %s: %s\n",
-		     app_name, command.common.name,
-		     error->message);
-
-	    g_error_free (error);
-
-	    goto out;
-	}
-
-	tp_proxy_prepare_async (a, NULL, account_ready, NULL);
+	command.common.account = prefix (command.common.account);
+	a = mc_account_new (daemon, command.common.account);
+	mc_account_call_when_ready (a, account_ready, NULL);
     }
-    g_object_unref (dbus);
+    g_object_unref (daemon);
 
     main_loop = g_main_loop_new (NULL, FALSE);
     g_main_loop_run (main_loop);
 
-out:
     if (am) g_object_unref (am);
     if (a) g_object_unref (a);
 
